package com.company.sample.view.adres;
import com.company.sample.entity.Adres;
import com.company.sample.view.main.MainView;
import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.Focusable;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.router.Route;
import io.jmix.core.DataManager;
import io.jmix.core.entity.KeyValueEntity;
import io.jmix.flowui.component.grid.DataGrid;
import io.jmix.flowui.component.grid.editor.DataGridEditor;
import io.jmix.flowui.data.grid.ContainerDataGridItems;
import io.jmix.flowui.model.CollectionContainer;
import io.jmix.flowui.model.DataContext;
import io.jmix.flowui.view.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


@Route(value = "adreses", layout = MainView.class)
@ViewController("Adres.list")
@ViewDescriptor("adres-list-view.xml")
@LookupComponent("adresesDataGrid")
@DialogMode(width = "64em")
public class AdresListView extends StandardListView<Adres>
{
    @ViewComponent
    private DataContext dataContext;

    @Autowired
    private DataManager dataManager;

    @ViewComponent
    private DataGrid<Adres> adresesDataGrid;

    @ViewComponent
    private CollectionContainer<Adres> adresesDc;

    @Subscribe
    public void onInit(final InitEvent event)
    {
        //	which fields we should list
        List<KeyValueEntity> kvEntities = dataManager.loadValues("select e.yflFieldPath, e.yflSort from TSysFieldList e where e.yflCategory = :cat order by e.yflSort")
                .store("main")
                .properties("yflFieldPath", "yflSort")
                .parameter("cat", 1)
                .list();

        if (!kvEntities.isEmpty()) {
            addColumnsToGrid(kvEntities);
            adresesDataGrid.setItems(new ContainerDataGridItems<>(adresesDc));
        }

        DataGridEditor<Adres> editor = adresesDataGrid.getEditor();
        adresesDataGrid.addItemDoubleClickListener(e -> {
            editor.editItem(e.getItem());
            Component editorComponent = e.getColumn().getEditorComponent();
            if (editorComponent instanceof Focusable) {
                ((Focusable<?>) editorComponent).focus();
            }
        });
    }

    protected void addColumnsToGrid(List<KeyValueEntity> kvEntities)
    {
        String column;
        for (KeyValueEntity kvEntity : kvEntities) {
            column = kvEntity.getValue("yflFieldPath");
            adresesDataGrid.addComponentColumn( adres -> { return new Span();} )
                        .setAutoWidth(true)
                        .setFlexGrow(0)
                        .setHeader(column)
                        .setKey(column)
                        .setResizable(true)
                        .setSortable(true);
        }
    }

    @Subscribe(target = Target.DATA_CONTEXT)
    public void onChange(DataContext.ChangeEvent event) {
        if ( dataContext.hasChanges() ) {
            dataContext.save();
        }
    }

}